BankBagSearcher = {}

local bbs = BankBagSearcher
bbs.version = "0.22a"
bbs.author = "Wukar"
bbs.name = "BankBagSearcher"
-- initially not filtered, maybe use a shallow copy function
bbs.attachTo = ZO_SharedRightPanelBackground
bbs.playerInventory = PLAYER_INVENTORY
bbs.data = {
    inventory = ZO_PlayerInventoryBackpack,
    playerbank = ZO_PlayerBankBackpack,
    guildbank = ZO_GuildBankBackpack,
}
bbs.activeBank = nil

local function BBS_SearchItemByName(inventory, name)
    local itemCount = #inventory.data
    local saveIndex = 1
    local filteredInventory = {}
    
    --d(inventory:GetName())
            
    if (name ~= "" and itemCount > 0) then
        for i, item in pairs(inventory.data) do
            local itemName = item.data.name:lower()
            if (itemName:find(name)) then
                filteredInventory[saveIndex] = item
                saveIndex = saveIndex + 1
            end
        end
        inventory.data = filteredInventory
    end
    ZO_ScrollList_Commit(inventory)
end


local function BBS_OnActionLayerPushed(layerIndex, activeLayerIndex)
    --d("BBS_OnShow Event")
    --if (bbs.data.inventory:IsHidden() == true) then
      --  return
    --end
    
    if (bbs.data.inventory:IsHidden() 
        and bbs.data.playerbank:IsHidden() 
        and bbs.data.guildbank:IsHidden()) 
    then
        return
    end
    
	bbs.editbox:SetText("")
    bbs.label:SetHidden(false)
        
    if (bbs.playerInventory:IsBanking()) then
        bbs.activeBank = "playerbank"
    elseif (bbs.playerInventory:IsGuildBanking()) then
        bbs.activeBank = "guildbank"
    else
        bbs.activeBank = nil
    end
end

local function BBS_SearchBoxOnTextChanged()
    --d("BBS_SearchBoxOnTextChanged Event")
    local text = bbs.editbox:GetText():lower()
        
	bbs.playerInventory:RefreshAllInventorySlots(INVENTORY_BACKPACK)
    BBS_SearchItemByName(bbs.data.inventory, text)
    
    if (bbs.activeBank ~= nil) then
        bbs.playerInventory:RefreshAllGuildBankItems()
        BBS_SearchItemByName(bbs.data[bbs.activeBank], text)
    end
    if (text:len() > 0) then
        bbs.label:SetHidden(true)        
    else
        bbs.label:SetHidden(false)
    end
end

local function BBS_AddSearchBox()
    local wm = WINDOW_MANAGER
    
	local bg = wm:CreateControlFromVirtual(bbs.name .. "Bg", bbs.attachTo, "ZO_EditBackdrop")
	bg:SetDimensions(300,28)
	bg:SetAnchor(LEFT, bbs.attachTo, BOTTOMLEFT, 30, 20)

	bbs.editbox = wm:CreateControlFromVirtual(bbs.name .. "SearchBox", bg, "ZO_DefaultEditForBackdrop")
	bbs.editbox:SetParent(bg)
	bbs.editbox:SetResizeToFitDescendents(false)
	bbs.editbox:SetMouseEnabled(true)
	bbs.editbox:SetText("")
    --bbs.editbox = editbox

	local label = wm:CreateControl(bbs.name .. "SearchBoxLabel", bbs.editbox, CT_LABEL)
	label:SetDimensions(200, 26)
	label:SetAnchor(LEFT)
	label:SetFont("ZoFontWinH4")
	label:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS)
	label:SetText("filter inventory")
    bbs.label = label
    
	bbs.editbox:SetHandler("OnTextChanged", BBS_SearchBoxOnTextChanged)
end

-- set asleep for now
--[[
local function BBS_GetActiveTabs(tabControl)
    local children = {}
    local counter = tabControl:GetNumChildren()
    for i=1, counter do
        local child = tabControl:GetChild(i)
        
        if (child:GetType() == CT_CONTROL) then
            -- add a handler for each control
            -- TODO: add one handler to the parent and handle tab changes
            child:SetHandler("OnMouseUp", function(self)
                BBS_SearchBoxOnTextChanged(bbs.editbox) 
            end)
            children[i] = child
        end
    end
    return children
end
]]

-- set asleep for now
--[[
local function BBS_AddTabEvents(tabControl)
    local activeTabs = BBS_GetActiveTabs(tabControl)
    bbs.tabs = activeTabs
    for _, tab in pairs(activeTabs) do
        d(tab:GetName() .. " " .. tab:GetType())
    end
end
]]

local function BBS_OnGlobalMouseUp(eventCode, button, ctrl, alt, _shift, command)
    -- use left clicks only
    if (button ~= 1) then return end
   
    local obj = WINDOW_MANAGER:GetMouseOverControl()
    if ( obj == nil or not obj:GetOwningWindow()) then return end
    
    local parentName = obj:GetParent():GetName()
    local searchBackpack = nil
    -- use player inventory so far
    if (parentName == "ZO_PlayerInventoryTabs") then
        searchBackpack = bbs.data.inventory
    elseif (parentName == "ZO_GuildBankTabs") then
        searchBackpack = bbs.data.guildbank
    elseif (parentName ~= "ZO_PlayerBankTabs")  then
        searchBackpack = bbs.data.bank
    end
    
    if (searchBackpack == nil) then return end;
    
    -- trigger the search again
    BBS_SearchBoxOnTextChanged(searchBackpack) 
end

local function BBS_OnOpenBank()
    bbs.activeBank = "playerbank"
end

local function BBS_OnOpenGuildBank()
   bbs.activeBank = "guildbank"
end

local function BBS_OnCloseBank()
    bbs.activeBank = nil
end

local function BBS_OnGuildBankItemsReady()
    -- TODO: use this to show the search filter
end

local function BBS_OnGuildBankItemChanged()
    BBS_SearchBoxOnTextChanged(bbs.data.guildbank)     
end

local function BBS_Initialize(_, addon)
	if (addon ~= "BankBagSearcher") then return end
        
    BBS_AddSearchBox()
        
    --BBS_AddTabEvents(ZO_PlayerInventoryTabs)
    -- use action layers as using the default "OnShow" removes the new item status
	--ZO_PlayerInventory:SetHandler("OnShow", BBS_OnShow)
	BankBagSearcherControl:RegisterForEvent(EVENT_ACTION_LAYER_PUSHED, BBS_OnActionLayerPushed)
    
	BankBagSearcherControl:RegisterForEvent(EVENT_GLOBAL_MOUSE_UP, BBS_OnGlobalMouseUp)
	BankBagSearcherControl:RegisterForEvent(EVENT_OPEN_BANK, BBS_OnOpenBank)
	BankBagSearcherControl:RegisterForEvent(EVENT_CLOSE_BANK, BBS_OnCloseBank)
	BankBagSearcherControl:RegisterForEvent(EVENT_OPEN_GUILD_BANK, BBS_OnOpenGuildBank)
	BankBagSearcherControl:RegisterForEvent(EVENT_CLOSE_GUILD_BANK, BBS_OnCloseBank)
	BankBagSearcherControl:RegisterForEvent(EVENT_GUILD_BANK_ITEM_REMOVED, BBS_OnGuildBankItemChanged)
	BankBagSearcherControl:RegisterForEvent(EVENT_GUILD_BANK_ITEM_ADDED, BBS_OnGuildBankItemChanged)
    --BankBagSearcherControl:RegisterForEvent(EVENT_GUILD_BANK_ITEMS_READY, BBS_OnGuildBankItemsReady)
end

-- Hook initialization onto the EVENT_ADD_ON_LOADED listener
EVENT_MANAGER:RegisterForEvent("BankBagSearcher", EVENT_ADD_ON_LOADED, BBS_Initialize)
